import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import controlP5.*; 
import processing.serial.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class monitorNTARSv1 extends PApplet {

/* ****************************************************************************
This program is monitoring program of NT-ARSv1 made by NTRexLAB.
The commercial web-page of NT-ARSv1 is http://ntrexgo.com
In NT-ARSv1, commands are as follows:
  <CAO> : to get data at once
  <CAH> : to get data continuously at 10ms intervals
  <CAE> : to stop recieving data.
And incomming data is organized as follows:
  <Roll angle, Pitch angle, Roll angular velocity, Pitch angular velocit>.
The NT-ARSv1's output data are multiplying 1000 times after expressed radians.   

                                                by PinkWink in NTRexLAB
**************************************************************************** */




ControlP5 cp5;
Serial arsPort;

DropdownList l;
ArsButtons pauseBt, stopBt, rollAngBt, pitchAngBt, rollAngVelBt, pitchAngVelBt;

int graphPanelXPos = 110;
int graphPanelYPos = 100;
int graphPanelXSize = 500;
int graphPanelYSize = 240;
int graphPanelYCenterPos = graphPanelYPos+graphPanelYSize/2; 
int panelGridHalfLength = 15;
int panelGridYDivision = graphPanelYSize/6;
int selectPanelXPos = graphPanelXPos;
int selectPanelYPos = graphPanelYPos+graphPanelYSize+40;
int selectPanelXSize = graphPanelXSize;
int selectPanelYSize = panelGridYDivision;
int buttonXStartPos = graphPanelXPos+60;
int buttonYPos = selectPanelYPos+selectPanelYSize/2;
int buttonXDiv = selectPanelXSize/5;

float resizingYSize = PApplet.parseFloat(graphPanelYSize)/180;
float resizingAngVelYSize = PApplet.parseFloat(graphPanelYSize)/1000;

float[] rollAng = new float[graphPanelXSize];
float[] pitchAng = new float[graphPanelXSize];
float[] rollAngVel = new float[graphPanelXSize];
float[] pitchAngVel = new float[graphPanelXSize];

String availablePort[];
String connectPort;

String[] graphYLabel = {"90", "60", "30", " 0", "-30", "-60", "-90"};

boolean arsConnection = false;

PFont fontText, bigfontText;

int ColorOfBackground = color(245,245,245);
int ColorOfListBoxBackground = color(190,190,190);
int ColorOfListBoxActive = color(150,150,150);
int ColorOfListBoxForeground = color(150,150,150);
int ColorOfListBoxSetColor = color(100,100,100);
int ColorOfRollAngLine = color(255,0,0);
int ColorOfPitchAngLine = color(0,255,0);
int ColorOfRollAngVelLine = color(255,100,0);
int ColorOfPitchAngVelLine = color(100,255,0);

public void setup() {
  size(650, 450);
  
  fontText = loadFont("CenturyGothic-12.vlw");
  bigfontText = loadFont("CenturyGothic-32.vlw");
  
  pauseBt = new ArsButtons(graphPanelXPos, graphPanelYPos-30, false, "rec");
  stopBt = new ArsButtons(graphPanelXPos+80, graphPanelYPos-30, false, "rec");
  rollAngBt = new ArsButtons(buttonXStartPos, buttonYPos, true, "ell");
  pitchAngBt = new ArsButtons(buttonXStartPos+buttonXDiv, buttonYPos, true, "ell");
  rollAngVelBt = new ArsButtons(buttonXStartPos+buttonXDiv*2, buttonYPos, false, "ell");
  pitchAngVelBt = new ArsButtons(buttonXStartPos+buttonXDiv*3, buttonYPos, false, "ell");
  
  cp5 = new ControlP5(this);
  l = cp5.addDropdownList("AvailablePorts");
    
  comPortList(l);
}

public void draw() {
  background(ColorOfBackground);
  drawGraphPanel();
  drawText();
  drawGraph();
}

public void mousePressed(){
  if(mouseButton==LEFT){
    if(rollAngBt.isButtonClicked(mouseX, mouseY, 10)) {
      rollAngBt.buttonClick(); }
    if(pitchAngBt.isButtonClicked(mouseX, mouseY, 10)) {
      pitchAngBt.buttonClick(); }
    if(rollAngVelBt.isButtonClicked(mouseX, mouseY, 10)) {
      rollAngVelBt.buttonClick(); }
    if(pitchAngVelBt.isButtonClicked(mouseX, mouseY, 10)) {
      pitchAngVelBt.buttonClick(); }
    if(pauseBt.isButtonClicked(mouseX, mouseY, 10)) {
      pauseBt.buttonClick(); }
    if(stopBt.isButtonClicked(mouseX, mouseY, 10)) {
      if(stopBt.buttonClicked) {
        stopBt.buttonClick();
        arsPort.write("<CAH>");
        delay(20);
      }else {
        stopBt.buttonClick(); 
        arsPort.write("<CAE>");
        delay(20);
      }
    }
  }
}

public void drawGraph() {
  noFill();  
  if (rollAngBt.buttonClicked) {
    stroke(ColorOfRollAngLine);
    strokeWeight(2);
    beginShape();
      for (int xPos=0; xPos<rollAng.length; xPos++) {
        float tmp = saturatingValue(rollAng[xPos], 90);        
        vertex(xPos+graphPanelXPos, graphPanelYCenterPos - tmp*resizingYSize);
      }
    endShape();
  }
  if (pitchAngBt.buttonClicked) {
    stroke(ColorOfPitchAngLine);
    strokeWeight(2);
    beginShape();
      for (int xPos=0; xPos<pitchAng.length; xPos++) {
        float tmp = saturatingValue(pitchAng[xPos], 90); 
        vertex(xPos+graphPanelXPos, graphPanelYCenterPos - tmp*resizingYSize);
      }
    endShape();
  }
  if (rollAngVelBt.buttonClicked) {
    stroke(ColorOfRollAngVelLine);
    strokeWeight(1);
    beginShape();
      for (int xPos=0; xPos<rollAngVel.length; xPos++) {
        float tmp = saturatingValue(rollAngVel[xPos], 500);
        vertex(xPos+graphPanelXPos, graphPanelYCenterPos - tmp*resizingAngVelYSize);
      }
    endShape();
  }
  if (pitchAngVelBt.buttonClicked) {
    stroke(ColorOfPitchAngVelLine);
    strokeWeight(1);
    beginShape();
      for (int xPos=0; xPos<pitchAngVel.length; xPos++) {
        float tmp = saturatingValue(pitchAngVel[xPos], 500);
        vertex(xPos+graphPanelXPos, graphPanelYCenterPos - tmp*resizingAngVelYSize);
      }
    endShape();
  }
}

public void serialEvent(Serial p) {
  String arsValues = "";
  
  arsValues = arsPort.readStringUntil(10);
  if (!pauseBt.buttonClicked && (arsValues != null)) {
      calAngles(arsValues);
  }  
}

public float saturatingValue(float target, float limitValue) {
  float resizingResult;
  float tmp = abs(target);
  if (tmp>limitValue) {
    resizingResult = target/tmp*limitValue;
  } else {
    resizingResult = target;
  }        
  return resizingResult;
}

public void calAngles(String s) {
  int lastPosInString = s.indexOf('>');
  s = s.substring(1, lastPosInString);
  
  int arsResultArray[] = PApplet.parseInt(split(s, ','));
  
  rollAng = subset(rollAng, 1);
  pitchAng = subset(pitchAng, 1);
  rollAngVel = subset(rollAngVel, 1);
  pitchAngVel = subset(pitchAngVel, 1);
  
  rollAng = append(rollAng, PApplet.parseFloat(arsResultArray[0])*0.001f*180/PI);
  pitchAng = append(pitchAng, PApplet.parseFloat(arsResultArray[1])*0.001f*180/PI);
  rollAngVel = append(rollAngVel, PApplet.parseFloat(arsResultArray[2])*0.001f*180/PI);
  pitchAngVel = append(pitchAngVel, PApplet.parseFloat(arsResultArray[3])*0.001f*180/PI);   
}

public void controlEvent(ControlEvent theEvent) {
  
  if (theEvent.isGroup() && theEvent.name().equals("AvailablePorts")){
    int connectPortNo = (int)theEvent.group().getValue();
    connectPort = availablePort[connectPortNo];
  }
  if (!arsConnection) { 
    arsPort = new Serial(this, connectPort, 115200); 
    arsConnection = true;
  }
  
  delay(100);  
  arsPort.write("<CAH>");    
}

public void comPortList(DropdownList ddl) {
  availablePort = arsPort.list();
  
  ddl.setPosition(10, 70);
  ddl.setSize(80, 60);
  ddl.setItemHeight(15);
  ddl.setBarHeight(15);
  ddl.setColorBackground(ColorOfListBoxBackground);
  ddl.setColorActive(ColorOfListBoxActive);
  ddl.setColorForeground(ColorOfListBoxForeground);
  ddl.setValue(0);
  ddl.captionLabel().toUpperCase(true);
  ddl.captionLabel().set("AvailablePorts");
  ddl.captionLabel().setColor(ColorOfListBoxSetColor);  
  ddl.captionLabel().style().marginTop = 3;
  ddl.valueLabel().style().marginTop = 3; 
  
  for (int i=0; i<availablePort.length; i++) {
    l.addItem(availablePort[i], i);
  }
}
class ArsButtons {
  int posX, posY;
  String type;
  boolean buttonClicked;    
  int ColorOfClickedButton = color(150,150,150);    
  
  ArsButtons(int tmpX, int tmpY, boolean initValue, String tmpType) {
    buttonClicked = initValue;
    posX = tmpX;
    posY = tmpY;
    type = tmpType;
    
    drawButton();
  }
  
  public boolean isButtonClicked(int X, int Y, int buttonArea) {
    boolean buttonClickedResult = false;
    
    if (type=="rec") {
      if(X<(posX+buttonArea) && X>posX && Y<(posY+buttonArea) && Y>posY) {
        buttonClickedResult = true; }
    } else if (type=="ell") {      
      if(X<(posX+buttonArea/2) && X>(posX-buttonArea/2) 
          && Y<(posY+buttonArea/2) && Y>(posY-buttonArea/2)) {
            buttonClickedResult = true; }
    }
    return buttonClickedResult;
  }
  
  public void buttonClick() {
    if (buttonClicked) {
      buttonClicked = false;
    } else {
      buttonClicked = true;
    }
  }  
  
  public void drawButton() {
    fill(ColorOfBackground);
    if (type=="rec") {
      rect(posX, posY, 10, 10);
      if (buttonClicked) { fill(ColorOfClickedButton); } 
      else { fill(ColorOfBackground); } 
      rect(posX+2, posY+2, 6, 6);
    } else if (type=="ell") {      
      ellipse(posX, posY, 10, 10);
      if (buttonClicked) { fill(ColorOfClickedButton); } 
      else { fill(ColorOfBackground); }
      ellipse(posX, posY, 6, 6);
    } 
  }
}
public void drawGraphPanel(){
  stroke(ColorOfListBoxForeground);
  strokeWeight(1.5f);
  fill(color(255,255,255));
  rect(graphPanelXPos, graphPanelYPos, graphPanelXSize, graphPanelYSize); 
  
  fill(ColorOfBackground);
  rect(selectPanelXPos, selectPanelYPos, selectPanelXSize, selectPanelYSize);
  
  pauseBt.drawButton();
  stopBt.drawButton();
  rollAngBt.drawButton();
  pitchAngBt.drawButton();
  rollAngVelBt.drawButton();
  pitchAngVelBt.drawButton();

  strokeWeight(1);
  for (int i=(graphPanelYPos+panelGridYDivision); 
              i<(graphPanelYPos+graphPanelYSize); i=i+panelGridYDivision) {
    for (int j=(graphPanelXPos+buttonXDiv);
              j<(graphPanelXPos+graphPanelXSize); j=j+buttonXDiv) {
      line(j+panelGridHalfLength, i, j-panelGridHalfLength, i);
      line(j,i+panelGridHalfLength, j, i-panelGridHalfLength);   
    }
  }
    
  int tmpX = graphPanelXPos+graphPanelXSize/10;
  for (int i=0; i<9; i++) {
    line(tmpX-panelGridHalfLength+i*graphPanelXSize/10, graphPanelYCenterPos, 
          tmpX+panelGridHalfLength+i*graphPanelXSize/10, graphPanelYCenterPos);
  }  
}

public void drawText() {
  textFont(bigfontText);
  fill(100);  
  text("NT-ARSv1 Monitor",10,35);
  textFont(fontText);
  text("This Processing code is monitoring program for NT-ARSv1",280,51);
  text("NT-ARSv1 Monitoring Ver. 0.80 by PinkWink in NTRexLAB.",280,63);
  //text("by PinkWink in http://pinkwink.kr/",420,75);
  text("x division = 1 second",
    graphPanelXPos+graphPanelXSize/2-50,graphPanelYPos+graphPanelYSize+15);
  for (int i=0; i<7; i++){
    text(graphYLabel[i], graphPanelXPos-20, graphPanelYPos+panelGridYDivision*i+5);
  }
  text("RollAngle", buttonXStartPos+11, buttonYPos+5);
  text("PitchAngle", buttonXStartPos+buttonXDiv+11, buttonYPos+5);
  text("RollAngVel", buttonXStartPos+buttonXDiv*2+11, buttonYPos+5);
  text("PitchAngVel", buttonXStartPos+buttonXDiv*3+11, buttonYPos+5);  
  text("Pause", graphPanelXPos+13, graphPanelYPos-20);
  text("STOP", graphPanelXPos+93, graphPanelYPos-20);    
}
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "monitorNTARSv1" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
